require 'csv'
require 'date'

require 'eaternet/lives_1_0/adapter'
require 'eaternet/lives_1_0/csv_parser'
require 'eaternet/loggable'
require 'eaternet/util'

module Eaternet
  module Agencies
    class Sf < Eaternet::Lives_1_0::Adapter
      include Eaternet::Loggable
      include Eaternet::Lives_1_0::CsvParser

      def businesses
        convert csv: 'businesses.csv', to_type: :business
      end

      def inspections
        convert csv: 'inspections.csv', to_type: :inspection
      end

      def violations
        convert csv: 'violations.csv', to_type: :violation
      end

      def feed_info
        Eaternet::Lives_1_0::FeedInfo.new do |fi|
          fi.feed_date =         Date.today
          fi.feed_version =      '1.0'
          fi.municipality_name = 'San Francisco'
          fi.municipality_url =  'https://www.sfdph.org/dph/EH/Food/Score/default.asp'
          fi.contact_email =     'contact@sfdph.org'
        end
      end

      def adapter_name
        'SF'
      end

      def zip_file_url
        'https://extxfer.sfdph.org/food/SFBusinesses.zip'
      end

      private

      def business(csv_row)
        Eaternet::Lives_1_0::Business.new do |b|
          csv_row.headers.each do |header|
            b.send("#{header.downcase}=", csv_row[header])
          end
        end
      end

      def inspection(csv_row)
        Eaternet::Lives_1_0::Inspection.new do |b|
          csv_row.headers.each do |header|
            b.send("#{header.downcase}=", csv_row[header])
            b.date = Date.parse(csv_row['date'])
            b.score = csv_row['score'].to_i
          end
        end
      end

      def violation(csv_row)
        Eaternet::Lives_1_0::Violation.new do |b|
          csv_row.headers.each do |header|
            b.send("#{header.downcase}=", csv_row[header])
            b.date = Date.parse(csv_row['date'])
            b.description = fix(description: csv_row['description'])
          end
        end
      end

      def fix(description:)
        # Remove bracketed violation facts, non-conforming to LIVES
        return description unless description =~ /^([^\[]+)/
        $1.strip
      end

      def csv_reader(path:)
        CSV.new(open(path, encoding: 'ISO-8859-1'), headers: true)
      end
    end
  end
end
