require 'date'

require 'eaternet/lives_1_0/csv_parser'
require 'eaternet/loggable'
require 'eaternet/util'
require 'eaternet/agencies/snhd_prototype'

module Eaternet
  module Agencies
    class Snhd < Eaternet::Lives_1_0::Adapter
      include Eaternet::Loggable
      include Eaternet::Lives_1_0::CsvParser

      def initialize
        @prototype = Eaternet::Agencies::SnhdPrototype.new
      end

      def adapter_name
        'SNHD'
      end

      def businesses
        @prototype.businesses
                  .map { |proto| try_to_create(:business, from_csv_row: proto) }
                  .compact
      end

      def inspections
        @prototype.inspections
                  .map { |proto| try_to_create(:inspection, from_csv_row: proto) }
                  .compact
      end

      def business(proto)
        Eaternet::Lives_1_0::Business.new do |b|
          b.business_id = proto.orig_key
          b.name =    proto.name
          b.address = proto.address
          b.city =    proto.city
          b.postal_code = proto.zipcode
          b.state = 'NV'
        end
      end

      def inspection(proto)
        Eaternet::Lives_1_0::Inspection.new do |i|
          i.business_id = proto.business_orig_key
          i.score = proto.demerits.nil? ? nil : 100 - proto.demerits.to_i
          i.date = Date.parse(proto.date)
        end
      end

      def violations
        @violations ||= _violations
      end

      def feed_info
        Eaternet::Lives_1_0::FeedInfo.new do |fi|
          fi.feed_date = Date.today
          fi.feed_version = '1.0'
          fi.municipality_name = 'Southern Nevada'
          fi.municipality_url = 'http://southernnevadahealthdistrict.org/restaurants/index.php'
          fi.contact_email = 'environmentalhealth@snhdmail.org'
        end
      end

      def legends
        []
      end

      def _violations
        violation_kinds = {}
        @prototype.violation_kinds.each do |vk|
          violation_kinds[vk.orig_key] = vk
        end

        result = []

        @prototype.inspections.each do |proto_inspection|
          proto_inspection.violations.each do |kind_id|
            begin
              result << Eaternet::Lives_1_0::Violation.new do |v|
                v.business_id = proto_inspection.business_orig_key
                v.date =        Date.parse(proto_inspection.date)
                v.code =        violation_kinds[kind_id].code
                v.description = violation_kinds[kind_id].description
              end
            rescue StandardError => e
              logger.debug "Error creating violation kind #{kind_id} for inspection #{proto_inspection.orig_key}: #{e}"
            end
          end
        end
        result.compact
      end
    end
  end
end
