module Eaternet
  module Lives_1_0
    # @abstract Subclass and override {#businesses}, {#inspections},
    #   and optionally {#violations}, {#feed_info}, and {#legends} to
    #   implement a custom Lives 1.0 data source adapter.
    class Adapter
      # @example Print the number of restaurants in SomeCity.
      #   some_city = Eaternet::SomeCity.new
      #   puts some_city.businesses.count
      #
      # @required Yes
      # @return [Enumerable<Business>]
      def businesses
        fail 'Override Adapter#businesses to return an Enumerable of Business'
      end

      # @example Compute the average inspection score for SomeCity.
      #   some_city = Eaternet::SomeCity.new
      #   sum = some_city.inspections
      #           .map(&:score)
      #           .reduce(0, :+)
      #   count = some_city.inspections.count
      #
      #   puts "Average inspection score: " + sum / count
      #
      # @required Yes
      # @return [Enumerable<Inspection>]
      def inspections
        fail 'Override Adapter#inspections to return an Enumerable of Inspection'
      end

      # @required No
      # @return [Enumerable<Violation>]
      def violations
        fail 'Optionally override this to return an Enumerable of Violation'
      end

      # @example Retrieve the name & URL of SomeCity's health agency.
      #   some_city = Eaternet::SomeCity.new
      #   info = some_city.feed_info
      #   puts info.municipality_name
      #   puts info.municipality_url
      #
      # @required No
      # @return [FeedInfo]
      def feed_info
        fail 'Optionally override this to return a FeedInfo'
      end

      # @required No
      # @return [Enumerable<Legend>]
      def legends
        fail 'Optionally override this to return an Enumerable of Legend'
      end
    end
  end
end
