module Eaternet
  module Lives_1_0
    module CsvParser
      def adapter_name
        fail 'Override #adapter_name with a name for log output, e.g. "SF"'
      end

      def zip_file_url
        fail 'Override #zip_file_url with the zip archive location'
      end

      def convert(csv:, to_type:)
        csv_map(csv) { |row| try_to_create to_type, from_csv_row: row }.compact
      end

      def map_csv
        CSV.new(File.open(table_file, encoding: 'utf-8'), headers: true)
           .lazy
           .map { |row| yield(row) }
      end

      def csv_map(filename)
        csv_rows(filename).lazy.map { |row| yield(row) }
      end

      def csv_rows(filename)
        csv_reader(path: File.join(zip_dir, filename))
      end

      def zip_dir
        Util.download_and_extract_zipfile(zip_file_url)
      end

      def try_to_create(entity_type, from_csv_row:)
        send(entity_type, from_csv_row)
      rescue ArgumentError => e
        logger.warn("#{adapter_name} LIVES adapter") do
          "Couldn't create a LIVES #{entity_type} from #{from_csv_row.inspect}: #{e}"
        end
        nil
      end
    end
  end
end
