require 'eaternet/validated_object'

module Eaternet
  module Lives_1_0
    # Contains info about the feed itself.
    #
    # @see http://www.yelp.com/healthscores#feed_info LIVES / Feed Information
    #   specification
    class FeedInfo < ValidatedObject
      # This regular expression for checking email addresses is based on one
      # from one of Ryan Bates’ excellent screencasts.
      # See http://railscasts.com/episodes/211-validations-in-rails-3
      EMAIL_REGEX = /\A([^@\s]+)@((?:[-a-z0-9]+\.)+[a-z]{2,})\z/i
      URL_REGEX = %r{\Ahttps?:/}

      # Date that the feed was generated.
      #
      # @required Yes
      # @return [Date]
      attr_accessor :feed_date
      validates     :feed_date, presence: true, type: Date

      # Version of the LIVES specification used to generate this feed.
      # For example ‘0.4.1’.
      #
      # @required Yes
      # @return [String]
      attr_accessor :feed_version
      validates     :feed_version, presence: true, type: String

      # Name of the municipality providing this feed. For example
      # ‘San Francisco’ or ‘Multnomah County’.
      #
      # @required Yes
      # @return [String]
      attr_accessor :municipality_name
      validates     :municipality_name, presence: true, type: String

      # URL of the publishing municipality’s website.
      #
      # @required No
      # @return [String]
      attr_accessor :municipality_url
      validates     :municipality_url,
                    type: String,
                    format: { with: URL_REGEX },
                    allow_nil: true

      # Email address of the person to contact regarding invalid data in
      # this feed.
      #
      # @required No
      # @return [String]
      attr_accessor :contact_email
      validates     :contact_email,
                    type: String,
                    format: { with: EMAIL_REGEX },
                    allow_nil: true
    end
  end
end
