require 'eaternet/validated_object'

module Eaternet
  module Lives_1_0
    # Legends are mappings from score ranges to human-readable descriptions of
    # those scores. Municipalities can use these to communicate the way scores
    # are traditionally presented. For example, 0-60 may map to ‘Fail’ or
    # 95-100 may map to ‘A+’. The provided ranges in {#minimum_score} and
    # {#maximum_score} must cover the entire 0-100 span without overlap. For
    # scores at the edge of two ranges, the greater range will be used. For
    # example, if 80-90 is a ‘B’ and 90-100 is an ‘A’, a score of 90 would be
    # an ‘A’. These are optional.
    #
    # @see http://www.yelp.com/healthscores#legend LIVES / Legend specification
    class Legend < ValidatedObject
      # Minimum score that can be classified with this description.
      #
      # @required Yes
      # @return [Integer]
      attr_accessor :minimum_score
      validates :minimum_score, inclusion: { in: (0..100) }

      # Maximum score that can be classified with this description.
      #
      # @required Yes
      # @return [Integer]
      attr_accessor :maximum_score
      validates :maximum_score, inclusion: { in: (0..100) }

      # Formatted version of the score in the format typically presented by the
      # municipality. For example ‘A’ or ‘Pass’.
      #
      # @required Yes
      # @return [String]
      attr_accessor :description
      validates :description, presence: true, type: String
    end
  end
end
