# coding: utf-8
require 'eaternet/validated_object'

module Eaternet
  module Lives_1_0
    # A container for all the Legends in the data set. Performs validation on
    # the whole set of Legends to ensure they cover the entire range of scores
    # and do not overlap, as specified:
    #
    # "The provided ranges [...] must cover the entire 0–100 span without
    # overlap. For scores at the edge of two ranges, the greater range will be
    # used."
    #
    # @see http://www.yelp.com/healthscores#legend LIVES / Score Legend
    #   specification
    # @private
    class LegendGroup < ValidatedObject
      attr_accessor :legends

      # Check that all the items in this LegendGroup:
      #
      #   1. Are of the class, Legend
      #   2. Cover the range of scores from 0-100 without overlap
      #
      # @todo: Refactor
      class ComprehensiveValidator < ActiveModel::EachValidator
        def validate_each(record, attribute, legends)
          scores = (0..100).to_a
          legends.each do |legend|
            unless legend.class == Legend
              record.errors.add attribute, 'must be a Legend'
              return
            end
            range = (legend.minimum_score..legend.maximum_score)
            range.each do |score|
              if scores.delete(score).nil?
                unless score == legend.minimum_score || score == legend.maximum_score
                  record.errors.add attribute, 'may not overlap'
                  return
                end
              end
            end
          end
          unless scores.empty?
            record.errors.add attribute, 'do not cover entire span from 0–100'
          end
        end
      end

      validates :legends, comprehensive: true
    end
  end
end
