require 'eaternet/validated_object'

module Eaternet
  module Lives_1_0
    # Contains information about specific violations.
    #
    # @required No
    # @see http://www.yelp.com/healthscores#violations LIVES / Violations
    #   specification
    class Violation < ValidatedObject
      # Unique identifier of the business for which this violation applies
      #
      # @required Yes
      # @return [String]
      attr_accessor :business_id
      validates :business_id,
                type: String,
                presence: true

      # Date of violation. This should correspond with the related inspection.
      #
      # @required Yes
      # @return [Date]
      attr_accessor :date
      validates :date,
                type: Date,
                presence: true

      # Code for the violation. It is recommended that this be based on the FDA
      # Food Code. However, municipalities can decide to use pre-existing codes
      # for this field.
      #
      # @required No
      # @return [String]
      attr_accessor :code
      validates :code,
                type: String,
                allow_nil: true

      # One line description of the violation.
      #
      # @required No
      # @return [String]
      attr_accessor :description
      validates :description,
                type: String,
                allow_nil: true
    end
  end
end
