# frozen_string_literal: true
module Eaternet
  # A mixin to add logging functionality to a class.
  module Loggable
    # @return [Logger] the configured logger singleton
    def logger
      @logger ||= create_logger
    end

    private

    def create_logger
      logger = Logger.new(ENV['EATERNET_LOG_FILE'] || $stderr)
      logger.datetime_format = '%Y-%m-%d %H:%M:%S'
      logger.progname        = 'Eaternet Adapter (Ruby)'
      logger.level           =  ENV['EATERNET_LOG_SEVERITY'] || Logger::ERROR
      logger
    end
  end
end
