module Eaternet
  # A first pass at a health scores mini-framework which
  # is normalized and provides some more information than
  # LIVES.
  #
  # @private
  module Prototype
    module AbstractAdapter
      # @return [Enumerable<BusinessData>]
      def businesses
        fail 'Override this to return an Enumerable of BusinessData'
      end

      # @return [Enumerable<InspectionData>]
      def inspections
        fail 'Override this to return an Enumerable of InspectionData'
      end

      # @return [Enumerable<ViolationData>]
      def violations
        fail 'Override this to return an Enumerable of ViolationData'
      end

      # @return [Enumerable<ViolationData>]
      def violation_kinds
        fail 'Override this to return an Enumerable of ViolationKindData'
      end
    end

    class BusinessData
      # @return [String]
      attr_reader :name, :address, :city, :zipcode, :orig_key

      #:nodoc:
      def initialize(name:, address:, city:, zipcode:, orig_key:)
        @name = name
        @address = address
        @city = city
        @zipcode = zipcode
        @orig_key = orig_key
      end

      def ==(other)
        @orig_key == other.orig_key
      end

      def eql?(other)
        self == other
      end

      def hash
        @orig_key.hash
      end

      # @return [String]
      def to_s
        "Business #{@orig_key}"
      end
    end

    #:nodoc:
    class InspectionData
      # @return [String]
      attr_reader :orig_key, :business_orig_key, :score, :date, :demerits, :violations

      def initialize(orig_key:, business_orig_key:, score:, date:, demerits: nil, violations: [])
        @orig_key = orig_key
        @business_orig_key = business_orig_key
        @score = score
        @date = date
        @demerits = demerits
        @violations = violations
      end

      def to_s
        "Inspection #{@orig_key}"
      end
    end

    #:nodoc:
    class ViolationData
      # @return [String]
      attr_reader :orig_key, :inspection_id, :violation_kind_id

      def initialize(orig_key:, inspection_id:, violation_kind_id:)
        @orig_key = orig_key
        @inspection_id = inspection_id
        @violation_kind_id = violation_kind_id
      end

      def to_s
        "Violation #{@orig_key}"
      end
    end

    #:nodoc:
    class ViolationKindData
      # @return [String]
      attr_reader :orig_key, :code, :demerits, :description

      def initialize(orig_key:, code:, demerits:, description:)
        @orig_key = orig_key
        @code = code
        @demerits = demerits
        @description = description
      end

      def to_s
        "ViolationKind #{@orig_key}"
      end
    end
  end
end
